/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "isku.h"
#include "roccat_device_sysfs.h"
#include <string.h>
#include <stdlib.h>

static gchar const * const actual_profile_name = "actual_profile";

guint isku_actual_profile_read(RoccatDevice *isku, GError **error) {
	guint result;
	gchar *string;

	string = roccat_device_sysfs_read(isku, actual_profile_name, 0, error);

	if (*error)
		return 0;

	result = strtoul(string, NULL, 10);
	g_free(string);
	return result;
}

gboolean isku_actual_profile_write(RoccatDevice *isku, guint profile_index, GError **error) {
	gboolean retval;
	gchar *string;

	g_assert(profile_index < ISKU_PROFILE_NUM);

	string = g_strdup_printf("%i", profile_index);
	retval = roccat_device_sysfs_write(isku, actual_profile_name, string, strlen(string), error);
	g_free(string);

	return retval;
}
