/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepure.h"
#include "roccat_device_sysfs.h"
#include <string.h>
#include <stdlib.h>

static gchar const * const actual_profile_name = "actual_profile";

guint konepure_actual_profile_read(RoccatDevice *konepure, GError **error) {
	guint result;
	KonepureActualProfile *profile;

	profile = (KonepureActualProfile *)roccat_device_sysfs_read(konepure, actual_profile_name, sizeof(KonepureActualProfile), error);
	if (*error)
		return 0;

	result = profile->profile_index;
	g_free(profile);
	return result;
}

gboolean konepure_actual_profile_write(RoccatDevice *konepure, guint profile_index, GError **error) {
	KonepureActualProfile profile;

	g_assert(profile_index < KONEPLUS_PROFILE_NUM);

	profile.command = KONEPLUS_COMMAND_ACTUAL_PROFILE;
	profile.size = sizeof(KonepureActualProfile);
	profile.profile_index = profile_index;

	return roccat_device_sysfs_write(konepure, actual_profile_name, (gchar const *)&profile, sizeof(KonepureActualProfile), error);
}
