/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitaryconfig_info_dialog.h"
#include "roccat_warning_dialog.h"
#include "konepuremilitary.h"

#define KONEPUREMILITARY_INFO_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONEPUREMILITARY_INFO_DIALOG_TYPE, KonepuremilitaryInfoDialogClass))
#define IS_KONEPUREMILITARY_INFO_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONEPUREMILITARY_INFO_DIALOG_TYPE))
#define KONEPUREMILITARY_INFO_DIALOG_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONEPUREMILITARY_INFO_DIALOG_TYPE, KonepuremilitaryInfoDialogPrivate))

typedef struct _KonepuremilitaryInfoDialogClass KonepuremilitaryInfoDialogClass;

struct _KonepuremilitaryInfoDialogClass {
	RoccatInfoDialogClass parent_class;
};

struct _KonepuremilitaryInfoDialogPrivate {
	GtkLabel *firmware_label;
};

G_DEFINE_TYPE(KonepuremilitaryInfoDialog, konepuremilitary_info_dialog, ROCCAT_INFO_DIALOG_TYPE);

static void set_firmware(KonepuremilitaryInfoDialog *info_dialog, RoccatDevice *device) {
	KonepuremilitaryInfoDialogPrivate *priv = info_dialog->priv;
	GError *local_error = NULL;
	KoneplusInfo *info;
	gchar *firmware;

	info = konepuremilitary_info_read(device, &local_error);
	if (!roccat_handle_error_dialog(GTK_WINDOW(info_dialog), "Can't read info:", &local_error)) {
		gtk_label_set_text(priv->firmware_label, NULL);
		return;
	}

	firmware = koneplus_firmware_version_to_string(info->firmware_version);
	gtk_label_set_text(priv->firmware_label, firmware);
	g_free(firmware);
	g_free(info);
}

GtkWidget *konepuremilitary_info_dialog_new(GtkWindow *parent, RoccatDevice *device) {
	KonepuremilitaryInfoDialog *info_dialog;

	info_dialog = KONEPUREMILITARY_INFO_DIALOG(g_object_new(KONEPUREMILITARY_INFO_DIALOG_TYPE,
			"transient-for", parent,
			NULL));

	set_firmware(info_dialog, device);

	return GTK_WIDGET(info_dialog);
}

static void konepuremilitary_info_dialog_init(KonepuremilitaryInfoDialog *info_dialog) {
	KonepuremilitaryInfoDialogPrivate *priv = KONEPUREMILITARY_INFO_DIALOG_GET_PRIVATE(info_dialog);
	info_dialog->priv = priv;

	priv->firmware_label = GTK_LABEL(gtk_label_new(NULL));
	roccat_info_dialog_add_line(ROCCAT_INFO_DIALOG(info_dialog), "Firmware version", GTK_WIDGET(priv->firmware_label));
}

static void konepuremilitary_info_dialog_class_init(KonepuremilitaryInfoDialogClass *klass) {
	g_type_class_add_private(klass, sizeof(KonepuremilitaryInfoDialogPrivate));
}
