#ifndef __ROCCAT_KONEPUREMILITARY_H__
#define __ROCCAT_KONEPUREMILITARY_H__

/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepure.h"

G_BEGIN_DECLS

#define USB_DEVICE_ID_ROCCAT_KONEPURE_MILITARY 0x2dbf

typedef struct _KonepuremilitaryLight KonepuremilitaryLight;
typedef struct _KonepuremilitaryProfileSettings KonepuremilitaryProfileSettings;

enum {
	KONEPUREMILITARY_CPI_LEVELS_NUM = 5,
	KONEPUREMILITARY_CPI_MIN = 100,
	KONEPUREMILITARY_CPI_MAX = 5000,
	KONEPUREMILITARY_CPI_STEP = 100,
	KONEPUREMILITARY_FIRMWARE_SIZE = 57336,
	KONEPUREMILITARY_FIRMWARE_CHUNK_SIZE = 1024,
	KONEPUREMILITARY_FIRMWARE_CHUNKS = 56,
	KONEPUREMILITARY_ANGLESNAP_MIN = 1,
	KONEPUREMILITARY_ANGLESNAP_MAX = 6,
	KONEPUREMILITARY_SENSOR_ALIGNMENT_RANGE = 30,
};

struct _KonepuremilitaryLight {
	guint8 index;
	guint8 red;
	guint8 blue;
	guint8 green;
} __attribute__ ((packed));

struct _KonepuremilitaryProfileSettings {
	guint8 command; /* KONEPLUS_COMMAND_PROFILE_SETTINGS */
	guint8 size; /* 0x1f */
	guint8 profile_index; /* range 0-4 */
	guint8 xy_sync;
	guint8 sensitivity_x; /* range 0x1-0xb */
	guint8 sensitivity_y; /* range 0x1-0xb */
	guint8 cpi_levels_enabled; /* bits 1-5 */

	 /* range 1-50 means 100-5000 cpi */
	guint8 cpi_levels_x[KONEPLUS_PROFILE_SETTING_CPI_LEVELS_NUM];
	guint8 cpi_startup_level; /* range 0-4 */
	guint8 cpi_levels_y[KONEPLUS_PROFILE_SETTING_CPI_LEVELS_NUM];
	guint8 advanced;
	guint8 polling_rate;
	guint8 light_enabled; /* bit 0 */
	guint8 light_effect_mode; /* does nothing */
	guint8 color_flow_effect;
	guint8 light_effect_type;
	guint8 light_effect_speed; /* range 1-3 */
	KonepuremilitaryLight light;
	guint8 angle_snapping_value; /* 0x01 - 0x06 */
	guint8 sensor_alignment_value; /* 0x00 - 0x3c */
} __attribute__ ((packed));

typedef enum {
	KONEPUREMILITARY_PROFILE_SETTINGS_ADVANCED_ANGLE_SNAP_BIT = 0,
	KONEPUREMILITARY_PROFILE_SETTINGS_ADVANCED_SENSOR_ALIGNMENT_BIT = 1,
} KonepuremilitaryProfileSettingsAdvanced;

typedef enum {
	KONEPUREMILITARY_RMP_ANGLESNAP_FLAG_OFF = 0,
	KONEPUREMILITARY_RMP_ANGLESNAP_FLAG_ON = 1,
} KonepuremilitaryRmpAnglesnapFlag;

typedef enum {
	KONEPUREMILITARY_RMP_SENSORALIGN_FLAG_OFF = 0,
	KONEPUREMILITARY_RMP_SENSORALIGN_FLAG_ON = 1,
} KonepuremilitaryRmpSensoralignFlag;

#define KONEPUREMILITARY_DBUS_SERVER_PATH "/org/roccat/Konepuremilitary"
#define KONEPUREMILITARY_DBUS_SERVER_INTERFACE "org.roccat.Konepuremilitary"

#define KONEPUREMILITARY_DEVICE_NAME "Kone Pure Military"

RoccatDevice *konepuremilitary_device_first(void);
RoccatDeviceScanner *konepuremilitary_device_scanner_new(void);

gchar *konepuremilitary_device_read(RoccatDevice *roccat_device, guint command, gssize length, GError **error);
gboolean konepuremilitary_device_write(RoccatDevice *roccat_device, gchar const *buffer, gssize length, GError **error);

KoneplusRmp *konepuremilitary_default_rmp(void);
KoneplusRmp *konepuremilitary_rmp_load_actual(guint profile_index);
void konepuremilitary_rmp_update_from_device(KoneplusRmp *rmp, RoccatDevice *device, guint profile_index);
KoneplusRmp *konepuremilitary_rmp_load(RoccatDevice *device, guint profile_index, GError **error);
gboolean konepuremilitary_rmp_save_actual(KoneplusRmp *rmp, guint profile_index, GError **error);
gboolean konepuremilitary_rmp_save(RoccatDevice *device, KoneplusRmp *rmp, guint profile_index, GError **error);
KoneplusRmp *konepuremilitary_rmp_load_save_after_reset(RoccatDevice *device, guint profile_index, GError **error);

KonepuremilitaryProfileSettings *konepuremilitary_rmp_to_profile_settings(KoneplusRmp *rmp);
void konepuremilitary_rmp_update_with_profile_settings(KoneplusRmp *rmp, KonepuremilitaryProfileSettings const *profile_settings);

RoccatKeyFile *konepuremilitary_configuration_load(void);
gboolean konepuremilitary_configuration_save(RoccatKeyFile *config, GError **error);

DBusGProxy *konepuremilitary_dbus_proxy_new(void);
gboolean konepuremilitary_dbus_emit_profile_data_changed_outside(DBusGProxy *proxy, guint profile_number);
gboolean konepuremilitary_dbus_emit_profile_data_changed_outside_instant(guint profile_number);
gboolean konepuremilitary_dbus_emit_profile_changed_outside(DBusGProxy *proxy, guint profile_number);
gboolean konepuremilitary_dbus_emit_profile_changed_outside_instant(guint profile_number);
gboolean konepuremilitary_dbus_emit_configuration_changed_outside(DBusGProxy *proxy);

guint konepuremilitary_actual_profile_read(RoccatDevice *konepuremilitary, GError **error);
gboolean konepuremilitary_actual_profile_write(RoccatDevice *konepuremilitary, guint profile_index, GError **error);

gboolean konepuremilitary_profile_buttons_write(RoccatDevice *konepuremilitary, guint profile_index, KonepureProfileButtons *profile_buttons, GError **error);
KonepureProfileButtons *konepuremilitary_profile_buttons_read(RoccatDevice *konepuremilitary, guint profile_index, GError **error);

gboolean konepuremilitary_profile_settings_write(RoccatDevice *konepuremilitary, guint profile_index, KonepuremilitaryProfileSettings *profile_settings, GError **error);
KonepuremilitaryProfileSettings *konepuremilitary_profile_settings_read(RoccatDevice *konepuremilitary, guint profile_index, GError **error);
gboolean konepuremilitary_profile_settings_equal(KonepuremilitaryProfileSettings const *left, KonepuremilitaryProfileSettings const *right);

gboolean konepuremilitary_reset(RoccatDevice *konepuremilitary, GError **error);
guint konepuremilitary_firmware_version_read(RoccatDevice *konepuremilitary, GError **error);
KoneplusInfo *konepuremilitary_info_read(RoccatDevice *konepuremilitary, GError **error);

gboolean konepuremilitary_select(RoccatDevice *konepuremilitary, guint profile_index, guint request, GError **error);
gboolean konepuremilitary_check_write(RoccatDevice *konepuremilitary, GError **error);

gboolean konepuremilitary_macro_write(RoccatDevice *konepuremilitary, guint profile_index, guint key_index, KoneplusMacro *macro, GError **error);
KoneplusMacro *konepuremilitary_macro_read(RoccatDevice *konepuremilitary, guint profile_index, guint key_index, GError **error);

gboolean konepuremilitary_talk_easyshift(RoccatDevice *konepuremilitary, guint8 state, GError **error);
gboolean konepuremilitary_talk_easyshift_lock(RoccatDevice *konepuremilitary, guint8 state, GError **error);
gboolean konepuremilitary_talk_easyaim(RoccatDevice *konepuremilitary, guint8 state, GError **error);

gboolean konepuremilitary_talkfx(RoccatDevice *konepuremilitary, guint32 effect, guint32 ambient_color, guint32 event_color, GError **error);
gboolean konepuremilitary_talkfx_off(RoccatDevice *konepuremilitary, GError **error);

gboolean konepuremilitary_firmware_write_0(RoccatDevice *roccat_device, GError **error);
gboolean konepuremilitary_firmware_write_1(RoccatDevice *roccat_device, GError **error);
gboolean konepuremilitary_firmware_write_2(RoccatDevice *roccat_device, GError **error);
gboolean konepuremilitary_firmware_write_3(RoccatDevice *roccat_device, guint16 number, guint8 const *chunk, GError **error);
gboolean konepuremilitary_firmware_write_4(RoccatDevice *roccat_device, guint8 const *chunk, GError **error);
gboolean konepuremilitary_firmware_write_f(RoccatDevice *roccat_device, GError **error);

void konepuremilitary_rmp_set_anglesnap_value(KoneplusRmp *rmp, guint value);
guint konepuremilitary_rmp_get_anglesnap_value(KoneplusRmp *rmp);
void konepuremilitary_rmp_set_anglesnap_flag(KoneplusRmp *rmp, guint value);
guint konepuremilitary_rmp_get_anglesnap_flag(KoneplusRmp *rmp);
void konepuremilitary_rmp_set_sensoralign_value(KoneplusRmp *rmp, guint value);
guint konepuremilitary_rmp_get_sensoralign_value(KoneplusRmp *rmp);
void konepuremilitary_rmp_set_sensoralign_flag(KoneplusRmp *rmp, guint value);
guint konepuremilitary_rmp_get_sensoralign_flag(KoneplusRmp *rmp);

guint konepuremilitary_dcu_get(RoccatDevice *konepuremilitary, GError **error);
gboolean konepuremilitary_dcu_try(RoccatDevice *konepuremilitary, guint new_dcu, GError **error);
gboolean konepuremilitary_dcu_cancel(RoccatDevice *konepuremilitary, guint old_dcu, GError **error);
gboolean konepuremilitary_dcu_accept(RoccatDevice *konepuremilitary, guint new_dcu, GError **error);

gboolean konepuremilitary_sensor_write_register(RoccatDevice *konepuremilitary, guint8 reg, guint8 value, GError **error);
guint8 konepuremilitary_sensor_read_register(RoccatDevice *konepuremilitary, guint8 reg, GError **error);


G_END_DECLS

#endif

