/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitary.h"
#include "roccat_device_hidraw.h"
#include "g_roccat_helper.h"

gboolean konepuremilitary_select(RoccatDevice *konepuremilitary, guint profile_index, guint request, GError **error) {
	KoneplusControl control;

	g_assert(profile_index < KONEPLUS_PROFILE_NUM);

	control.command = KONEPLUS_COMMAND_CONTROL;
	control.value = profile_index;
	control.request = request;

	return konepuremilitary_device_write(konepuremilitary, (gchar const *)&control, sizeof(KoneplusControl), error);
}

gboolean konepuremilitary_check_write(RoccatDevice *konepuremilitary, GError **error) {
	KoneplusControl *control;
	guint8 value;

	do {
		g_usleep(100 * G_ROCCAT_USEC_PER_MSEC);
		control = (KoneplusControl *)konepuremilitary_device_read(konepuremilitary, KONEPLUS_COMMAND_CONTROL, sizeof(KoneplusControl), error);
		if (!control)
			return FALSE;
		value = control->value;
		g_free(control);
		switch (value) {
		case KONEPLUS_CONTROL_REQUEST_STATUS_OK:
			return TRUE;
			break;
		case KONEPLUS_CONTROL_REQUEST_STATUS_BUSY:
			g_usleep(500 * G_ROCCAT_USEC_PER_MSEC);
			break;
		case KONEPLUS_CONTROL_REQUEST_STATUS_CRITICAL:
			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "Write check got critical status");
			return FALSE;
			break;
		case KONEPLUS_CONTROL_REQUEST_STATUS_INVALID:
			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "Write check got invalid status");
			return FALSE;
			break;
		default:
			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_FAILED, "Write check unknown error");
			return FALSE;
			break;
		}
	} while(1);
}
