/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitary.h"
#include <unistd.h>

static KoneplusSensor *konepuremilitary_dcu_read(RoccatDevice *konepuremilitary, GError **error) {
	return (KoneplusSensor *)konepuremilitary_device_read(konepuremilitary, KONEPLUS_COMMAND_SENSOR, sizeof(KoneplusSensor), error);
}

static gboolean konepuremilitary_dcu_write(RoccatDevice *konepuremilitary, KoneplusSensor const *data, GError **error) {
	return konepuremilitary_device_write(konepuremilitary, (char const *)data, sizeof(KoneplusSensor), error);
}

guint konepuremilitary_dcu_get(RoccatDevice *konepuremilitary, GError **error) {
	KoneplusSensor *sensor;
	guint retval;

	sensor = konepuremilitary_dcu_read(konepuremilitary, error);
	if (!sensor)
		return 0;

	retval = sensor->dcu;
	g_free(sensor);
	return retval;
}

gboolean konepuremilitary_dcu_try(RoccatDevice *konepuremilitary, guint new_dcu, GError **error) {
	KoneplusSensor sensor;

	sensor.command = KONEPLUS_COMMAND_SENSOR;
	sensor.size = sizeof(KoneplusSensor);
	sensor.dcu = new_dcu;
	sensor.tcu = 0xff;
	sensor.median = 0xff;
	sensor.action = KONEPLUS_SENSOR_ACTION_UNDEFINED;

	return konepuremilitary_dcu_write(konepuremilitary, &sensor, error);
}

gboolean konepuremilitary_dcu_cancel(RoccatDevice *konepuremilitary, guint old_dcu, GError **error) {
	KoneplusSensor sensor;

	sensor.command = KONEPLUS_COMMAND_SENSOR;
	sensor.size = sizeof(KoneplusSensor);
	sensor.dcu = old_dcu;
	sensor.tcu = 0xff;
	sensor.median = 0xff;
	sensor.action = KONEPLUS_SENSOR_ACTION_CANCEL;

	return konepuremilitary_dcu_write(konepuremilitary, &sensor, error);
}

gboolean konepuremilitary_dcu_accept(RoccatDevice *konepuremilitary, guint new_dcu, GError **error) {
	KoneplusSensor sensor;

	sensor.command = KONEPLUS_COMMAND_SENSOR;
	sensor.size = sizeof(KoneplusSensor);
	sensor.dcu = new_dcu;
	sensor.tcu = 0xff;
	sensor.median = 0xff;
	sensor.action = KONEPLUS_SENSOR_ACTION_ACCEPT;

	return konepuremilitary_dcu_write(konepuremilitary, &sensor, error);
}
