/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitary.h"
#include <stdlib.h>

static void konepuremilitary_info_finalize(KoneplusInfo *info) {
	info->command = KONEPLUS_COMMAND_INFO;
	info->size = sizeof(KoneplusInfo);
}

static gboolean konepuremilitary_info_write(RoccatDevice *konepuremilitary, KoneplusInfo *info, GError **error) {
	konepuremilitary_info_finalize(info);
	return konepuremilitary_device_write(konepuremilitary, (gchar const *)info, sizeof(KoneplusInfo), error);
}

gboolean konepuremilitary_reset(RoccatDevice *konepuremilitary, GError **error) {
	KoneplusInfo info = { };
	info.firmware_version = KONEPLUS_INFO_FUNCTION_RESET;
	return konepuremilitary_info_write(konepuremilitary, &info, error);
}

guint konepuremilitary_firmware_version_read(RoccatDevice *konepuremilitary, GError **error) {
	KoneplusInfo *info;
	guint result;

	info = konepuremilitary_info_read(konepuremilitary, error);
	if (info) {
		result = info->firmware_version;
		g_free(info);
		return result;
	}

	return 0;
}

KoneplusInfo *konepuremilitary_info_read(RoccatDevice *konepuremilitary, GError **error) {
	return (KoneplusInfo *)konepuremilitary_device_read(konepuremilitary, KONEPLUS_COMMAND_INFO, sizeof(KoneplusInfo), error);
}
