/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitary.h"
#include "koneplus_rmp.h"
#include "roccat_helper.h"
#include <string.h>

static guint8 konepuremilitary_macro_calc_checksum(KoneplusMacro const *macro) {
	return ROCCAT_BYTESUM_PARTIALLY(macro, KoneplusMacro, command, checksum);
}

static void konepuremilitary_macro_set_checksum(KoneplusMacro *macro) {
	macro->checksum = GUINT16_TO_LE(konepuremilitary_macro_calc_checksum(macro));
}

static void konepuremilitary_macro_finalize(KoneplusMacro *macro, guint profile_index, guint key_index) {
	g_assert(profile_index < KONEPLUS_PROFILE_NUM);
	macro->command = KONEPLUS_COMMAND_MACRO;
	koneplus_macro_set_size(macro, sizeof(KoneplusMacro));
	macro->profile_number = profile_index;
	macro->button_number = key_index;
	konepuremilitary_macro_set_checksum(macro);
}

gboolean konepuremilitary_macro_write(RoccatDevice *konepuremilitary, guint profile_index, guint key_index, KoneplusMacro *macro, GError **error) {
	g_assert(profile_index < KONEPLUS_PROFILE_NUM);
	konepuremilitary_macro_finalize(macro, profile_index, key_index);
	return konepuremilitary_device_write(konepuremilitary, (gchar const *)macro, sizeof(KoneplusMacro), error);
}

KoneplusMacro *konepuremilitary_macro_read(RoccatDevice *konepuremilitary, guint profile_index, guint key_index, GError **error) {
	KoneplusMacro *macro;

	roccat_device_lock(konepuremilitary);

	if (!konepuremilitary_select(konepuremilitary, profile_index, key_index, error)) {
		roccat_device_unlock(konepuremilitary);
		return NULL;
	}

	macro = (KoneplusMacro *)konepuremilitary_device_read(konepuremilitary, KONEPLUS_COMMAND_MACRO, sizeof(KoneplusMacro), error);

	roccat_device_unlock(konepuremilitary);

	return macro;
}
