/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitary.h"
#include "roccat_helper.h"

gboolean konepuremilitary_profile_buttons_write(RoccatDevice *konepuremilitary, guint profile_index, KonepureProfileButtons *profile_buttons, GError **error) {
	g_assert(profile_index < KONEPLUS_PROFILE_NUM);
	konepure_profile_buttons_finalize(profile_buttons, profile_index);
	return konepuremilitary_device_write(konepuremilitary, (gchar const *)profile_buttons, sizeof(KonepureProfileButtons), error);
}

KonepureProfileButtons *konepuremilitary_profile_buttons_read(RoccatDevice *konepuremilitary, guint profile_index, GError **error) {
	KonepureProfileButtons *profile_buttons;

	g_assert(profile_index < KONEPLUS_PROFILE_NUM);

	roccat_device_lock(konepuremilitary);

	if (!konepuremilitary_select(konepuremilitary, profile_index, KONEPLUS_CONTROL_REQUEST_PROFILE_BUTTONS, error)) {
		roccat_device_unlock(konepuremilitary);
		return NULL;
	}

	profile_buttons = (KonepureProfileButtons *)konepuremilitary_device_read(konepuremilitary, KONEPLUS_COMMAND_PROFILE_BUTTONS, sizeof(KonepureProfileButtons), error);

	roccat_device_unlock(konepuremilitary);
	return profile_buttons;
}
