/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "konepuremilitary.h"

static gboolean konepuremilitary_tcu_write(RoccatDevice *konepuremilitary, KoneplusTcu *tcu, GError **error) {
	return konepuremilitary_device_write(konepuremilitary, (gchar const *)tcu, sizeof(KoneplusTcu), error);
}

static KoneplusTcu *konepuremilitary_tcu_read(RoccatDevice *konepuremilitary, GError **error) {
	return (KoneplusTcu *)konepuremilitary_device_read(konepuremilitary, KONEPLUS_COMMAND_TCU, sizeof(KoneplusTcu), error);
}

static gboolean konepuremilitary_tcu_write_struct(RoccatDevice *konepuremilitary, guint8 action, guint8 reg, guint8 value, GError **error) {
	KoneplusTcu tcu;

	tcu.command = KONEPLUS_COMMAND_TCU;
	tcu.action = action;
	tcu.reg = reg;
	tcu.value = value;

	return konepuremilitary_tcu_write(konepuremilitary, &tcu, error);
}

gboolean konepuremilitary_sensor_write_register(RoccatDevice *konepuremilitary, guint8 reg, guint8 value, GError **error) {
	return konepuremilitary_tcu_write_struct(konepuremilitary, KONEPLUS_TCU_ACTION_WRITE, reg, value, error);
}

guint8 konepuremilitary_sensor_read_register(RoccatDevice *konepuremilitary, guint8 reg, GError **error) {
	KoneplusTcu *tcu;
	guint8 result;

	konepuremilitary_tcu_write_struct(konepuremilitary, KONEPLUS_TCU_ACTION_READ, reg, 0, error);
	if (*error)
		return 0;

	tcu = konepuremilitary_tcu_read(konepuremilitary, error);
	if (*error)
		return 0;
	result = tcu->value;
	g_free(tcu);

	return result;
}
