/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ryos_control.h"
#include "ryos_device.h"
#include "g_roccat_helper.h"
#include <string.h>

gboolean ryos_select(RoccatDevice *ryos, guint profile_index, guint request, GError **error) {
	RyosControl control;
	gboolean retval;

	g_assert(profile_index < RYOS_PROFILE_NUM);

	control.command = RYOS_COMMAND_CONTROL;
	control.value = profile_index;
	control.request = request;

	retval = ryos_device_write(ryos, (gchar const *)&control, sizeof(RyosControl), error);

	/* FIXME Ignoring inapropriate INVALID or CRITICAL answers as long
	 * as firmware requires it
	 */
	if ((request == 0xb2 || request == 0xe0) && retval == FALSE) {
		g_clear_error(error);
		return TRUE;
	}

	return retval;
}

gboolean ryos_check_write(RoccatDevice *ryos, GError **error) {
	RyosControl *control;
	guint8 value;

	do {
		g_usleep(10 * G_ROCCAT_USEC_PER_MSEC);
		control = (RyosControl *)ryos_device_read(ryos, RYOS_COMMAND_CONTROL, sizeof(RyosControl), error);
		if (!control)
			return FALSE;
		value = control->value;
		g_free(control);
		switch (value) {
		case RYOS_CONTROL_VALUE_STATUS_OK:
			return TRUE;
			break;
		case RYOS_CONTROL_VALUE_STATUS_BUSY:
			g_usleep(10 * G_ROCCAT_USEC_PER_MSEC);
			break;
		case RYOS_CONTROL_VALUE_STATUS_CRITICAL:
			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "Write check got critical status");
			return FALSE;
			break;
		case RYOS_CONTROL_VALUE_STATUS_INVALID:
			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "Write check got invalid status");
			return FALSE;
			break;
		default:
			g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_FAILED, "Write check unknown error");
			return FALSE;
			break;
		}
	} while(1);
}
