/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ryos_control.h"
#include "ryos_device.h"
#include "ryos.h"
#include "roccat.h"
#include "roccat_device_hidraw.h"

static gchar const * const driver_name = "hidraw";

static guint const device_ids[4] = { USB_DEVICE_ID_ROCCAT_RYOS_MK,
	USB_DEVICE_ID_ROCCAT_RYOS_MK_GLOW,
	USB_DEVICE_ID_ROCCAT_RYOS_MK_PRO,
	0 };

RoccatDevice *ryos_device_first(void) {
	return roccat_device_first(driver_name, device_ids, ROCCAT_USB_INTERFACE_PROTOCOL_NONE);
}

RoccatDeviceScanner *ryos_device_scanner_new(void) {
	return roccat_device_scanner_new(driver_name, device_ids, ROCCAT_USB_INTERFACE_PROTOCOL_NONE);
}

gchar *ryos_device_read(RoccatDevice *roccat_device, guint command, gssize length, GError **error) {
	return roccat_device_hidraw_read(roccat_device, command, length, error);
}

gboolean ryos_device_write(RoccatDevice *roccat_device, gchar const *buffer, gssize length, GError **error) {
	gboolean retval;

	retval = roccat_device_hidraw_write(roccat_device, buffer, length, error);
	if (!retval)
		return FALSE;

	return ryos_check_write(roccat_device, error);
}
